import random

from constants import extension
from constants import collections as extension_collections


from perxis.extensions.actions import make_action_dict
from perxis.extensions import extension_service_pb2
from perxis.extensions.extension_service import ExtensionService
from perxis.extensions import extension_pb2
from perxis.spaces import spaces_pb2
from perxis.environments import environments_pb2
from perxis.collections import helpers as collections_helpers
from perxis.roles import roles_pb2
from perxis.common import common_pb2
from perxis.clients import clients_pb2


class Servicer(ExtensionService):
    extension_id = extension.ID
    collections = collections_helpers.make_collection_instances("./schemes", extension_collections.schemes_mapping)
    actions = [
        make_action_dict(
            extension_id=extension.ID,
            action_id="demo-action-no-callback",
            name="Действие с несуществующим каллбэком",
            kind=extension_pb2.Action.Kind.ITEMS,
            classes=[extension_collections.TEST_COLLECTION_ID]
        ),
        make_action_dict(
            extension_id=extension.ID,
            action_id="demo-action-items",
            name="Действие над элементами",
            kind=extension_pb2.Action.Kind.ITEMS,
            classes=[extension_collections.TEST_COLLECTION_ID]
        ),
        make_action_dict(
            extension_id=extension.ID,
            action_id="demo-action-item",
            name="Действие над одним элементом",
            kind=extension_pb2.Action.Kind.ITEM,
            classes=[extension_collections.TEST_COLLECTION_ID]
        )
    ]
    roles = [
        roles_pb2.Role(
            id="demo-role",
            description="Описание к роли",
            rules=[
                common_pb2.Rule(
                    collection_id="dealers_cities",
                    actions=[common_pb2.CREATE, common_pb2.UPDATE, common_pb2.DELETE],
                )
            ],
            environments=["*"],
            allow_management=False,
        )
    ]
    clients = [
        clients_pb2.Client(
            id="demo-client",
            name="Демонстрационный клиент",
            description="Описание созданного расширением клиента",
            role_id="demo-role",
            api_key={
                "rotate": True
            }
        )
    ]

    async def action_demo_action_items(
            self,
            request: extension_pb2.ActionRequest,
            context
    ) -> extension_service_pb2.ActionResponse:
        self.logger.info(
            "Действие над несколькими элементами %s %s %s %s" % (
                request.space_id, request.env_id, request.item_ids, request.collection_id
            )
        )

        message = await self.spaces_service.Get(
            spaces_pb2.GetRequest(
                space_id=request.space_id
            )
        )

        space = message.space

        message = await self.environments_service.Get(
            environments_pb2.GetRequest(
                space_id=request.space_id,
                env_id=request.env_id,
            )
        )

        environment = message.env

        with_error = bool(random.randint(0, 1))

        if with_error:
            state = extension_service_pb2.ActionResponse.State.ERROR
        else:
            state = extension_service_pb2.ActionResponse.State.DONE

        title = f"Действие над одним элементом в пространстве {space.name} и окружении {', '.join(environment.aliases)}"

        return extension_service_pb2.ActionResponse(
            state=state,
            title=title,
            error=", ".join(["Ошибка"] if with_error else []),
            msg=f"{'Ошибка' if with_error else 'ОК'} ({request.item_ids}, {request.collection_id})"
        )

    async def action_demo_action_item(
            self,
            request: extension_pb2.ActionRequest,
            context
    ) -> extension_service_pb2.ActionResponse:
        self.logger.info(
            "Действие над одним элементом %s %s %s %s" % (
                request.space_id, request.env_id, request.item_id, request.collection_id
            )
        )

        with_error = bool(random.randint(0, 1))

        if with_error:
            state = extension_service_pb2.ActionResponse.State.ERROR
        else:
            state = extension_service_pb2.ActionResponse.State.DONE

        return extension_service_pb2.ActionResponse(
            state=state,
            title="Действие над одним элементом",
            error=", ".join(["Ошибка"] if with_error else []),
            msg=f"{'Ошибка' if with_error else 'ОК'} ({request.item_id}, {request.collection_id})"
        )
